<?php

class KokenWatermark extends KokenPlugin {

	private $tmps = array();

	function __construct()
	{

		$this->database_fields = array(
			'content' => array(
				'koken_watermark_data' => array(
					'type' => 'VARCHAR',
					'constraint' => 255,
					'null' => true,
					'default' => '{"watermark":0}'
				)
			)
		);

		$this->register_filter('darkroom.render.imagemagick', 'imagemagick');
		$this->register_filter('darkroom.render.gd', 'gd');
		$this->register_filter('darkroom.render.imagick', 'imagick');
		$this->register_filter('api.content.before', 'stamper');
		$this->register_filter('api.content', 'revert_stamper');
		$this->register_hook('darkroom.render.complete', 'cleanup');
		$this->register_filter('api.content.update', 'update');
	}

	function confirm_setup()
	{
		$this->clear_image_cache();
		touch($this->get_storage_path() . '/' . $this->data->upload);
		return true;
	}

	function after_install()
	{
		$this->clear_image_cache();
		touch($this->get_storage_path() . '/' . $this->data->upload);
		return true;
	}

	function after_upgrade()
	{
		$this->clear_image_cache();
		touch($this->get_storage_path() . '/' . $this->data->upload);
		return true;
	}

	function update($data)
	{
		if ($this->data->public && (isset($data['visibility']) ||  $this->data->threshold === 'some'))
		{
			$data['file_modified_on'] = time();
			$this->clear_image_cache($data['id']);
		}
		return $data;
	}

	function after_uninstall()
	{
		$this->clear_image_cache();
		return true;
	}

	function cleanup($path)
	{
		foreach($this->tmps as $file)
		{
			unlink($file);
		}
	}

	/*
		Changing watermarks may not show up b/c of the browser cache.
		So, here we will modify the file_modified_on timestamp to that
		of the watermark (if it is newer) so that the cache buster
		will change and force a new version of the image.
	*/
	function stamper($data)
	{
		$mark = filemtime($this->get_storage_path() . '/' . $this->data->upload);
		if ($mark > $data['file_modified_on']['timestamp'])
		{
			$data['file_modified_on']['timestamp_cache'] = $data['file_modified_on']['timestamp'];
			$data['file_modified_on']['timestamp'] = $mark;
		}
		return $data;
	}

	/*
		Revert the stamp back so API data is not changed.
	*/
	function revert_stamper($data)
	{
		if (isset($data['file_modified_on']['timestamp_cache']))
		{
			$data['file_modified_on']['timestamp'] = $data['file_modified_on']['timestamp_cache'];
			unset($data['file_modified_on']['timestamp_cache']);
		}
		return $data;
	}

	private function get_position_bit()
	{
		$map = array(
			"northwest",
			"north",
			"northeast",
			"west",
			"center",
			"east",
			"southwest",
			"south",
			"southeast",
		);

		return array_search($this->data->position, $map);
	}

	private function get_position($ww, $wh, $iw, $ih)
	{
		$key = $this->get_position_bit();

		if (in_array($key, array(0,3,6))) {
			$x = 0;
		} elseif (in_array($key, array(1,4,7))) {
			$x = ($iw/2) - ($ww/2);
		} else {
			$x = $iw - $ww;
		}

		if (in_array($key, array(0,1,2))) {
			$y = 0;
		} elseif (in_array($key, array(3,4,5))) {
			$y = ($ih/2) - ($wh/2);
		} else {
			$y = $ih - $wh;
		}

		if ($this->data->padding > 0 && $this->data->position !== 'center')
		{
			if (strpos($this->data->position, 'north') !== false)
			{
				$y += $this->data->padding;
			}
			else if (strpos($this->data->position, 'south') !== false)
			{
				$y -= $this->data->padding;
			}

			if (strpos($this->data->position, 'west') !== false)
			{
				$x += $this->data->padding;
			}
			else if (strpos($this->data->position, 'east') !== false)
			{
				$x -= $this->data->padding;
			}
		}

		return array($x, $y);
	}

	private function verify($options, $img_data)
	{
		$max = max($options['width'], $options['height']);
		$watermarkData =  json_decode($img_data['koken_watermark_data']) ?: new stdClass;

		if ($options['retina']) {
			$max /= 2;
		}

		$this->data->padding = ($options['width'] / 2) * ($this->data->padding / 100);
		return $this->data->threshold !== 'none'
			&& $img_data['visibility'] !== 'private'
			&& (!$this->data->public || (isset($img_data['visibility']) && $img_data['visibility']['raw'] === 'public'))
			&& ($this->data->threshold === 'all' || $this->data->threshold <= $max)
			&& ($this->data->threshold !== 'some' || $watermarkData->watermark );
	}

	private function watermark_size($ww, $wh, $options)
	{
		if ($this->data->size === 'proportional')
		{
			$wa = $ww / $wh;
			$ww = $options['width'];
			if (!$this->data->tile)
			{
				$ww -= $this->data->padding*2;
			}
			$ww *= $this->data->scale/100;
			$wh = $ww / $wa;
		}

		return array($ww, $wh);
	}

	function tiledata($ww, $wh, $options)
	{
		$midpointx = $options['width'] / 2;
		$midpointy = $options['height'] / 2;

		$ww += $this->data->padding*2;
		$wh += $this->data->padding*2;

		$offsetw = $ww - ($midpointx % $ww) + ($ww / 2);
		$offseth = $wh - ($midpointy % $wh) + ($wh / 2);

		$tilesizew = $options['width'] + $offsetw;
		$tilesizeh = $options['height'] + $offseth;

		return array($tilesizew, $tilesizeh, $offsetw, $offseth, $ww, $wh);
	}

	function imagemagick($command, $options, $img_data)
	{
		if ($this->verify($options, $img_data))
		{
			preg_match('/.*convert/', $command, $matches);
			$convert_path = $matches[0];
			$composite = str_replace('convert', 'composite', $convert_path);

			preg_match_all('/\-limit (threads|map|memory) \d+/', $command, $matches);
			$water_limit = join(' ', $matches[0]);

			if (!empty($water_limit))
			{
				$water_limit = ' ' . $water_limit;
			}

			$water_path = $this->get_storage_path() . '/' . $this->data->upload;

			list($ww, $wh) = getimagesize($water_path);
			list($ww, $wh) = $this->watermark_size($ww, $wh, $options);

			if ($this->data->tile)
			{
				list($tilesizew, $tilesizeh, $offsetw, $offseth, $tilew, $tileh) = $this->tiledata($ww, $wh, $options);

				$tmp = dirname(__FILE__) . '/' . uniqid() . $this->data->upload;
				$command = $convert_path . " $water_path -background none -resize {$ww}x{$wh} -gravity center -extent {$tilew}x{$tileh} - | " . $convert_path . " -background none -size {$tilesizew}x{$tilesizeh} tile:- $tmp && " . $command;
				$command .=	 " - | $composite{$water_limit} -dissolve {$this->data->opacity} -geometry {$tilesizew}x{$tilesizeh}-{$offsetw}-{$offseth} $tmp -";
				$this->tmps[] = $tmp;
			}
			else
			{
				$geometry = "{$ww}x{$wh}";

				if ($this->data->padding > 0 && $this->data->position !== 'center')
				{
					$inset_x = 0;
					$inset_y = 0;

					if (strpos($this->data->position, 'north') !== false || strpos($this->data->position, 'south') !== false)
					{
						$inset_y = $this->data->padding;
					}

					if (strpos($this->data->position, 'west') !== false || strpos($this->data->position, 'east') !== false)
					{
						$inset_x = $this->data->padding;
					}

					$geometry .= "+{$inset_x}+{$inset_y}";
				} else {
					$geometry .= '+0+0';
				}

				$command .= " - | $composite{$water_limit} -dissolve {$this->data->opacity} -gravity {$this->data->position} -geometry $geometry $water_path -";
			}
		}
		return $command;
	}

	function imagick($image, $options, $img_data)
	{
		if ($this->verify($options, $img_data))
		{
			$watermark = new Imagick($this->get_storage_path() . '/' . $this->data->upload);

			$ww = $watermark->getImageWidth();
			$wh = $watermark->getImageHeight();

			list($ww, $wh) = $this->watermark_size($ww, $wh, $options);
			$watermark->resizeImage($ww, $wh, imagick::FILTER_LANCZOS, 1, true);

			$watermark->setResourceLimit(6, 1); // Threads
			$watermark->setResourceLimit(IMagick::RESOURCETYPE_MAP, 33554432);
			$watermark->setResourceLimit(IMagick::RESOURCETYPE_MEMORY, 67108864);

			if ($this->data->tile)
			{
				list($tilesizew, $tilesizeh, $offsetw, $offseth, $tilew, $tileh) = $this->tiledata($ww, $wh, $options);
				$container = new Imagick();
				$container->newImage($tilew, $tileh, 'none');
				$container->compositeImage($watermark, Imagick::COMPOSITE_DISSOLVE, ($tilew - $ww) / 2, ($tileh - $wh) / 2);
				$tile = new Imagick();
				$tile->newImage($tilesizew, $tilesizeh, 'none');
				$watermark = $tile->textureImage($container);

				$container->clear();
				$tile->clear();

				$x = -$offsetw;
				$y = -$offseth;
			}
			else
			{
				list($x, $y) = $this->get_position(
					$ww,
					$wh,
					$image->getImageWidth(),
					$image->getImageHeight()
				);
			}

			$watermark->evaluateImage(Imagick::EVALUATE_MULTIPLY, $this->data->opacity/100, Imagick::CHANNEL_ALPHA);
			$image->compositeImage($watermark, Imagick::COMPOSITE_DISSOLVE, $x, $y);
			$watermark->clear();
		}

		return $image;
	}

	function gd($image, $options, $img_data)
	{
		if ($this->verify($options, $img_data))
		{
			$watermark = $this->get_storage_path() . '/' . $this->data->upload;
			if (file_exists($watermark))
			{
				$info = pathinfo($watermark);

				if (strtolower($info['extension']) === 'png')
				{
					if (imagetypes() & IMG_PNG) {
						$water_img = imagecreatefrompng($watermark);
					}
				}
				else
				{
					if (imagetypes() & IMG_GIF) {
						$water_img = imagecreatefromgif($watermark);
					}
				}
			}

			if (isset($water_img)) {
				$original_ww = imagesx($water_img);
				$original_wh = imagesy($water_img);

				list($ww, $wh) = $this->watermark_size($original_ww, $original_wh, $options);

				if ($this->data->tile)
				{
					list($tilesizew, $tilesizeh, $offsetw, $offseth, $tilew, $tileh) = $this->tiledata($ww, $wh, $options);

					$water_img_resized = imagecreatetruecolor($tilew, $tileh);
					imagealphablending($water_img_resized, false);
					imagesavealpha($water_img_resized, true);
					imagefilledrectangle($water_img_resized, 0, 0, $tilew, $tileh, imagecolorallocatealpha($water_img_resized, 0, 0, 0, 127));
					imagecopyresized($water_img_resized, $water_img, ($tilew - $ww) / 2, ($tileh - $wh) / 2, 0, 0, $ww, $wh, $original_ww, $original_wh);

					$tile = imagecreatetruecolor($tilesizew, $tilesizeh);
					imagealphablending($tile, false);
					imagefilledrectangle($tile, 0, 0, $tilesizew-1, $tilesizeh-1, imagecolorallocatealpha($tile, 0, 0, 0, 127));
					imagesettile($tile, $water_img_resized);
					imagefilledrectangle($tile, 0, 0, $tilesizew-1, $tilesizeh-1, IMG_COLOR_TILED);

					if ($this->data->opacity == 100) {
						imagecopy($image, $tile, -$offsetw, -$offseth, 0, 0, $tilesizew, $tilesizeh);
					} else {
						$bg_cut = imagecreatetruecolor($tilesizew, $tilesizeh);
						imagecopy($bg_cut, $image, 0, 0, -$offsetw, -$offseth, $tilesizew, $tilesizeh);
						imagecopy($image, $tile, -$offsetw, -$offseth, 0, 0, $tilesizew, $tilesizeh);
						imagecopymerge($image, $bg_cut, -$offsetw, -$offseth, 0, 0, $tilesizew, $tilesizeh, 100-$this->data->opacity);
						imagedestroy($bg_cut);
					}
					imagedestroy($tile);
					imagedestroy($water_img_resized);
				}
				else
				{
					list($x, $y) = $this->get_position(
						$ww,
						$wh,
						imagesx($image),
						imagesy($image)
					);

					if ($this->data->opacity == 100) {
						imagecopyresampled($image, $water_img, $x, $y, 0, 0, $ww, $wh, $original_ww, $original_wh);
					} else {
						$bg_cut = imagecreatetruecolor($ww, $wh);
						imagecopy($bg_cut, $image, 0, 0, $x, $y, $ww, $wh);
						imagecopyresampled($image, $water_img, $x, $y, 0, 0, $ww, $wh, $original_ww, $original_wh);
						imagecopymerge($image, $bg_cut, $x, $y, 0, 0, $ww, $wh, 100-$this->data->opacity);
						imagedestroy($bg_cut);
					}
				}
				imagedestroy($water_img);
			}
		}

		return $image;
	}

}