<?php

class KokenRating extends KokenPlugin {

	function __construct()
	{
		$this->lightbox = false;
		$this->database_fields = array(
			'content' => array(
				'koken_rating_count' => array(
					'type' => 'INT',
					'constraint' => 11,
					'unsigned' => true,
					'default' => 0,
				),
				'koken_rating_average' => array(
					'type' => 'FLOAT',
					'constraint' => 4,
					'unsigned' => true,
					'null' => true,
				)
			),
			'albums' => array(
				'koken_rating' => array(
					'type' => 'TINYINT',
					'constraint' => 1,
					'unsigned' => true,
					'default' => 0,
				)
			)
		);

		$this->register_filter('site.api_data', 'filter_api_data');
		$this->register_hook('content.listing', 'listing');
		$this->register_hook('before_closing_head', 'render_css');
		$this->register_hook('before_closing_body', 'render_js');
		$this->register_hook('rating', 'render_div');

		$this->register_hook('before_closing_head', 'set_lightbox');
		$this->register_hook('before_pjax', 'set_lightbox');
	}

	function filter_api_data($data)
	{
		if (isset($data['album']))
		{
			$this->album_show_rating = $data['album']['koken_rating'];
		}
		else if (isset($data['context']['album']))
		{
			$this->album_show_rating = $data['context']['album']['koken_rating'];
		}

		return $data;
	}

	function listing($content, $options)
	{

		if (isset($options['koken_rating'])) {
			$content->where('koken_rating_count >', 0);
		}
	}

	function set_lightbox($data)
	{
		$this->lightbox = $data['lightbox'];
	}

	function render_div($item)
	{
		$average = 0;
		$count = 0;
		$scope = $this->data->scope;

		if ($scope === 'none' || ($scope === 'some' && empty($this->album_show_rating))) {
			return;
		}

		if (isset($item['koken_rating_count']))
		{
			$count = $item['koken_rating_count'];
		}

		if (isset($item['koken_rating_average']))
		{
			$average = $item['koken_rating_average'];
		}

		$css_classes = $this->lightbox ? 'icon-heart ibutton icon-large' : 'kicon-heart';

		echo '<span class="koken-rating ' . $css_classes . '" data-cid="' . $item['id'] . '" data-average="' . $average . '" data-count="' . $count . '"></span>';
	}

	function render_css()
	{
		echo <<<OUT
<style type="text/css">
.koken-rating {
	cursor: pointer;
}
.koken-rating::after {
	content: attr(data-count);
}
.koken-rating[data-count="0"]::after {
	content: '';
}
</style>
OUT;
	}

	function render_js()
	{
		echo <<<OUT
<script type="text/javascript">
(function() {
	var ep = '{$this->get_path()}/site/rate.php';

	$('body').on('click', '.koken-rating', function(evt) {
		evt.preventDefault();

		var elm = $(this);
		var cid = elm.data('cid');

		$.post(ep, {
			cid: cid,
			rating: 1
		}, function(rating) {
			elm.attr({
				'data-count': rating.count,
				'data-average': rating.average
			});
		});
	});
})();
</script>
OUT;

	}
}
